@@ -4,15 +4,16 @@ use proc_macro2::{Ident, TokenStream};
 use quote::{format_ident, quote};
 use syn::{
     ext::IdentExt, visit_mut::VisitMut, AttributeArgs, Error, FnArg, ImplItem, ImplItemMethod,
-    ItemImpl, Pat, Path, ReturnType,
+    ItemImpl, Pat, Path, ReturnType, Type,
 };
 
 use crate::{
-    common_args::{APIMethod, DefaultValue, ExternalDocument},
+    common_args::{APIMethod, DefaultValue, ExternalDocument, ExtraHeader},
     error::GeneratorResult,
     utils::{
         convert_oai_path, get_crate_name, get_description, get_summary_and_description,
-        optional_literal, parse_oai_attrs, remove_description, remove_oai_attrs, RemoveLifetime,
+        optional_literal, optional_literal_string, parse_oai_attrs, remove_description,
+        remove_oai_attrs, RemoveLifetime,
     },
     validators::Validators,
 };
@@ -25,6 +26,10 @@ struct APIArgs {
     prefix_path: Option<SpannedValue<String>>,
     #[darling(default, multiple, rename = "tag")]
     common_tags: Vec<Path>,
+    #[darling(default, multiple, rename = "response_header")]
+    response_headers: Vec<ExtraHeader>,
+    #[darling(default, multiple, rename = "request_header")]
+    request_headers: Vec<ExtraHeader>,
 }
 
 #[derive(FromMeta)]
@@ -41,6 +46,10 @@ struct APIOperation {
     operation_id: Option<String>,
     #[darling(default)]
     external_docs: Option<ExternalDocument>,
+    #[darling(default, multiple, rename = "response_header")]
+    response_headers: Vec<ExtraHeader>,
+    #[darling(default, multiple, rename = "request_header")]
+    request_headers: Vec<ExtraHeader>,
 }
 
 #[derive(FromMeta, Default)]
@@ -76,6 +85,7 @@ pub(crate) fn generate(
     };
     let crate_name = get_crate_name(api_args.internal);
     let ident = item_impl.self_ty.clone();
+    let (impl_generics, _, where_clause) = item_impl.generics.split_for_impl();
     let mut ctx = Context {
         add_routes: Default::default(),
         operations: Default::default(),
@@ -133,7 +143,7 @@ pub(crate) fn generate(
     let expanded = quote! {
         #item_impl
 
-        impl #crate_name::OpenApi for #ident {
+        impl #impl_generics #crate_name::OpenApi for #ident #where_clause {
             fn meta() -> ::std::vec::Vec<#crate_name::registry::MetaApi> {
                 ::std::vec![#crate_name::registry::MetaApi {
                     paths: ::std::vec![#(#paths),*],
@@ -169,6 +179,8 @@ fn generate_operation(
         transform,
         operation_id,
         external_docs,
+        response_headers,
+        request_headers,
     } = args;
     let http_method = method.to_http_method();
     let fn_ident = &item_method.sig.ident;
@@ -304,7 +316,7 @@ fn generate_operation(
         });
 
         // param meta
-        let param_desc = optional_literal(&param_description);
+        let param_desc = optional_literal_string(&param_description);
         let deprecated = operation_param.deprecated;
         params_meta.push(quote! {
             if <#arg_ty as #crate_name::ApiExtractor>::TYPE == #crate_name::ApiExtractorType::Parameter {
@@ -318,7 +330,7 @@ fn generate_operation(
                 };
 
                 let meta_param = #crate_name::registry::MetaOperationParam {
-                    name: #param_name,
+                    name: ::std::string::ToString::to_string(#param_name),
                     schema: original_schema.merge(patch_schema),
                     in_type: <#arg_ty as #crate_name::ApiExtractor>::param_in().unwrap(),
                     description: #param_desc,
@@ -391,6 +403,58 @@ fn generate_operation(
         None => quote!(::std::option::Option::None),
     };
 
+    // extra request headers
+    let mut update_extra_request_headers = Vec::new();
+    for header in api_args.request_headers.iter().chain(&request_headers) {
+        let name = header.name.to_uppercase();
+        let description = optional_literal_string(&header.description);
+        let ty = match syn::parse_str::<Type>(&header.ty) {
+            Ok(ty) => ty,
+            Err(_) => return Err(Error::new(header.ty.span(), "Invalid type").into()),
+        };
+        let deprecated = header.deprecated;
+
+        update_extra_request_headers.push(quote! {
+            params.push(#crate_name::registry::MetaOperationParam {
+                name: ::std::string::ToString::to_string(#name),
+                schema: <#ty as #crate_name::types::Type>::schema_ref(),
+                in_type: #crate_name::registry::MetaParamIn::Header,
+                description: #description,
+                required: <#ty as #crate_name::types::Type>::IS_REQUIRED,
+                deprecated: #deprecated,
+            });
+        });
+    }
+
+    // extra response headers
+    let mut update_extra_response_headers = Vec::new();
+    for (idx, header) in api_args
+        .response_headers
+        .iter()
+        .chain(&response_headers)
+        .enumerate()
+    {
+        let name = header.name.to_uppercase();
+        let description = optional_literal_string(&header.description);
+        let ty = match syn::parse_str::<Type>(&header.ty) {
+            Ok(ty) => ty,
+            Err(_) => return Err(Error::new(header.ty.span(), "Invalid type").into()),
+        };
+        let deprecated = header.deprecated;
+
+        update_extra_response_headers.push(quote! {
+            for resp in &mut meta.responses {
+                resp.headers.insert(#idx, #crate_name::registry::MetaHeader {
+                    name: ::std::string::ToString::to_string(#name),
+                    description: #description,
+                    required: <#ty as #crate_name::types::Type>::IS_REQUIRED,
+                    deprecated: #deprecated,
+                    schema: <#ty as #crate_name::types::Type>::schema_ref(),
+                });
+            }
+        });
+    }
+
     ctx.operations.entry(oai_path).or_default().push(quote! {
         #crate_name::registry::MetaOperation {
             tags: ::std::vec![#(#tag_names),*],
@@ -400,6 +464,7 @@ fn generate_operation(
             external_docs: #external_docs,
             params: {
                 let mut params = ::std::vec::Vec::new();
+                #(#update_extra_request_headers)*
                 #(#params_meta)*
                 params
             },
@@ -408,7 +473,11 @@ fn generate_operation(
                 #(#request_meta)*
                 request
             },
-            responses: <#res_ty as #crate_name::ApiResponse>::meta(),
+            responses: {
+                let mut meta = <#res_ty as #crate_name::ApiResponse>::meta();
+                #(#update_extra_response_headers)*
+                meta
+            },
             deprecated: #deprecated,
             security: {
                 let mut security = ::std::vec![];
